@@ -39,6 +39,7 @@
 
 #define STYLESHEET DATADIR"/yelp/xslt/mal2html.xsl"
 #define MALLARD_NS "http://projectmallard.org/1.0/"
+#define MALLARD_CACHE_NS "http://projectmallard.org/cache/1.0/"
 
 #define YELP_MALLARD_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_MALLARD, YelpMallardPriv))
 
@@ -73,7 +74,8 @@ struct _YelpMallardPriv {
     GSList        *pending;
 
     xmlDocPtr      cache;
-    xmlNsPtr       cache_ns;
+    xmlNsPtr       cache_mal_ns;
+    xmlNsPtr       cache_cache_ns;
     GHashTable    *pages_hash;
 };
 
@@ -166,11 +168,13 @@ mallard_init (YelpMallard *mallard)
     priv->transforms_running = 0;
 
     priv->cache = xmlNewDoc (BAD_CAST "1.0");
-    priv->cache_ns = xmlNewNs (NULL, BAD_CAST MALLARD_NS, BAD_CAST "mal");
-    cur = xmlNewDocNode (priv->cache, priv->cache_ns, BAD_CAST "cache", NULL);
+    priv->cache_cache_ns = xmlNewNs (NULL, BAD_CAST MALLARD_CACHE_NS, BAD_CAST "cache");
+    priv->cache_mal_ns = xmlNewNs (NULL, BAD_CAST MALLARD_NS, BAD_CAST "mal");
+    cur = xmlNewDocNode (priv->cache, priv->cache_cache_ns, BAD_CAST "cache", NULL);
     xmlDocSetRootElement (priv->cache, cur);
-    priv->cache_ns->next = cur->nsDef;
-    cur->nsDef = priv->cache_ns;
+    priv->cache_cache_ns->next = priv->cache_mal_ns;
+    priv->cache_mal_ns->next = cur->nsDef;
+    cur->nsDef = priv->cache_cache_ns;
     priv->pages_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                               NULL,
                                               (GDestroyNotify) mallard_page_data_free);
@@ -373,10 +377,14 @@ mallard_try_run (YelpMallard *mallard,
 {
     /* We expect to be in a locked mutex when this function is called. */
     MallardPageData *page_data;
+    YelpError *error;
 
     page_data = g_hash_table_lookup (mallard->priv->pages_hash, page_id);
     if (page_data == NULL) {
-        printf ("FIXME: page not found\n");
+	error = yelp_error_new (_("Page not found"),
+				_("The page %s was not found in the document %s."),
+				page_id, mallard->priv->directory);
+	yelp_document_error_page (YELP_DOCUMENT (mallard), page_id, error);
         return;
     }
 
@@ -390,7 +398,7 @@ static void
 mallard_page_data_walk (MallardPageData *page_data)
 {
     xmlParserCtxtPtr parserCtxt = NULL;
-    xmlChar *id;
+    xmlChar *id = NULL;
 
     if (page_data->cur == NULL) {
         parserCtxt = xmlNewParserCtxt ();
@@ -411,7 +419,7 @@ mallard_page_data_walk (MallardPageData *page_data)
             goto done;
 
         page_data->cache = xmlNewChild (page_data->cache,
-                                        page_data->mallard->priv->cache_ns,
+                                        page_data->mallard->priv->cache_mal_ns,
                                         page_data->cur->name,
                                         NULL);
 
@@ -425,7 +433,7 @@ mallard_page_data_walk (MallardPageData *page_data)
         }
 
         info = xmlNewChild (page_data->cache,
-                            page_data->mallard->priv->cache_ns,
+                            page_data->mallard->priv->cache_mal_ns,
                             BAD_CAST "info", NULL);
         page_data->link_title = FALSE;
         page_data->sort_title = FALSE;
@@ -438,14 +446,14 @@ mallard_page_data_walk (MallardPageData *page_data)
             else if (xmlStrEqual (child->name, BAD_CAST "title")) {
                 xmlNodePtr node;
                 xmlNodePtr title_node = xmlNewChild (page_data->cache,
-                                                     page_data->mallard->priv->cache_ns,
+                                                     page_data->mallard->priv->cache_mal_ns,
                                                      BAD_CAST "title", NULL);
                 for (node = child->children; node; node = node->next) {
                     xmlAddChild (title_node, xmlCopyNode (node, 1));
                 }
                 if (!page_data->link_title) {
                     xmlNodePtr title_node = xmlNewChild (info,
-                                                         page_data->mallard->priv->cache_ns,
+                                                         page_data->mallard->priv->cache_mal_ns,
                                                          BAD_CAST "title", NULL);
                     xmlSetProp (title_node, BAD_CAST "type", BAD_CAST "link");
                     for (node = child->children; node; node = node->next) {
@@ -454,7 +462,7 @@ mallard_page_data_walk (MallardPageData *page_data)
                 }
                 if (!page_data->sort_title) {
                     xmlNodePtr title_node = xmlNewChild (info,
-                                                         page_data->mallard->priv->cache_ns,
+                                                         page_data->mallard->priv->cache_mal_ns,
                                                          BAD_CAST "title", NULL);
                     xmlSetProp (title_node, BAD_CAST "type", BAD_CAST "sort");
                     for (node = child->children; node; node = node->next) {
@@ -494,12 +502,8 @@ mallard_page_data_info (MallardPageData *page_data,
         else if (xmlStrEqual (child->name, BAD_CAST "title")) {
             xmlNodePtr node, title_node;
             xmlChar *type, *role;
-            title_node = xmlNewChild (cache_node,
-                                      page_data->mallard->priv->cache_ns,
-                                      BAD_CAST "title", NULL);
-            for (node = child->children; node; node = node->next) {
-                xmlAddChild (title_node, xmlCopyNode (node, 1));
-            }
+            title_node = xmlCopyNode (child, 1);
+            xmlAddChild (cache_node, title_node);
 
             type = xmlGetProp (child, BAD_CAST "type");
             role = xmlGetProp (child, BAD_CAST "role");
@@ -508,15 +512,6 @@ mallard_page_data_info (MallardPageData *page_data,
                 page_data->link_title = TRUE;
             if (xmlStrEqual (type, BAD_CAST "sort"))
                 page_data->sort_title = TRUE;
-
-            if (type) {
-                xmlSetProp (title_node, BAD_CAST "type", BAD_CAST type);
-                xmlFree (type);
-            }
-            if (role) {
-                xmlSetProp (title_node, BAD_CAST "role", BAD_CAST type);
-                xmlFree (role);
-            }
         }
         else if (xmlStrEqual (child->name, BAD_CAST "desc") ||
                  xmlStrEqual (child->name, BAD_CAST "link")) {
@@ -546,6 +541,7 @@ mallard_page_data_run (MallardPageData *page_data)
     yelp_transform_start (page_data->transform,
 			  page_data->xmldoc,
 			  params);
+    g_strfreev (params);
 }
 
 static void
